/*
    BLUES - BD-Java emulation server

    Copyright (C) 2007-2016 GuinpinSoft inc <blues@makemkv.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
package blues;

import java.io.PrintStream;

public class Log {
    public static final int LOG_ALL = 0;
    public static final int LOG_ERROR = 1;
    public static final int LOG_OUTPUT = 2;
    public static final int LOG_STACK = 3;
    public static final int LOG_PSR = 4;
    public static final int LOG_GPR = 5;
    public static final int LOG_SYS = 6;
    public static final int LOG_FILE = 7;
    public static final int LOG_LIBBLURAY = 8;
    public static final int LOG_EXCEPTIONS = 9;
    public static final int LOG_JAIL = 10;
    public static final int LOG_OUT = 11;
    public static final int LOG_ERR = 12;
    public static final int LOG_NET = 13;
    public static final int LOG_REFLECT = 14;
    public static final int LOG_MAX = 15;

    private static final int FRAME_ALWAYS = 0x8000;

    private static PrintStream out = null;
    private static boolean mask[] = new boolean[LOG_MAX];

    public static boolean enabled(int source) {
        if (source >= LOG_MAX)
            throw new IllegalArgumentException();
        if ((mask[LOG_ALL] == false) && (mask[source] == false)) {
            return false;
        }
        if (out == null)
            return false;
        return true;
    }

    public static synchronized void setOut(PrintStream aout) {
        out = aout;
    }

    public static synchronized void close() {
        if (out != null) {
            out.flush();
            out.close();
            out = null;
        }
    }


    public static synchronized void setEnabled(int source, boolean value) {
        if (source >= LOG_MAX)
            throw new IllegalArgumentException();
        mask[source] = value;
    }

    public static synchronized void log(int source, String msg) {
        if (enabled(source) == false)
            return;
        String[] strings = new String[] { msg };
        log(mask[LOG_STACK] ? 1 : -1, source, strings);
    }

    public static synchronized void log(int frame, int source, String msg) {
        if (enabled(source) == false)
            return;
        String[] strings = new String[] { msg };
        log(mask[LOG_STACK] ? 1 : -1, source, strings);
    }

    public static synchronized void logN(boolean error, String file, int line, String msg) {
        if (enabled(LOG_LIBBLURAY) == false)
            return;
        StringBuffer buf = new StringBuffer(msg.length() + file.length() + 32);
        if (error) {
            buf.append("ERR ");
        }
        buf.append(msg);
        buf.append(" ");
        buf.append(file);
        buf.append(":");
        buf.append(Integer.toString(line));
        log(LOG_LIBBLURAY, buf.toString());
    }

    public static synchronized void log(int source, String... strings) {
        log(mask[LOG_STACK] ? 1 : -1, source, strings);
    }

    public static synchronized void log(int frame, int source, String... strings) {
        if (enabled(source) == false)
            return;

        int size = 2 + 128;
        for (int i = 0; i < strings.length; i++) {
            if (strings[i] != null) {
                size += strings[i].length();
            } else {
                size += 7;
            }
        }
        StringBuffer buf = new StringBuffer(size);
        for (int i = 0; i < strings.length; i++) {
            if (strings[i] != null) {
                buf.append(strings[i]);
            } else {
                buf.append("<null>");
            }
        }

        boolean stack = mask[LOG_STACK];
        if ((frame & FRAME_ALWAYS) != 0) {
            stack = true;
            frame -= FRAME_ALWAYS;
        }

        if (stack && (frame >= 0)) {
            StackTraceElement[] traces = Thread.currentThread().getStackTrace();
            int offset = 2 + frame;
            int count = traces.length - offset;
            if (count > 3)
                count = 3;
            for (int i = 0; i < count; i++) {
                if (i == 0) {
                    buf.append(" @ ");
                } else {
                    buf.append(" ");
                }
                buf.append(traces[i + offset].toString());
            }
        }

        out.println(buf.toString());
    }

    public static synchronized void log(int source, String[] tail, String... strings) {
        log(mask[LOG_STACK] ? 1 : -1, source, tail, strings);
    }

    public static synchronized void log(int frame, int source, String[] tail, String... strings) {
        if (enabled(source) == false)
            return;
        String[] all = new String[strings.length + tail.length];
        System.arraycopy(strings, 0, all, 0, strings.length);
        System.arraycopy(tail, 0, all, strings.length, tail.length);
        log(frame,source, all);
    }

    public static void unimplemented() {
        log(FRAME_ALWAYS | 0, LOG_ERROR, "unimplemented");
    }
}
